__author__ = "Ilya Baldin"
__version__ = "0.1"
__maintainer__ = "Ilya Baldin"

import hashlib, base64
from Crypto.PublicKey import RSA
import requests
import urllib3.exceptions

"""set of helper functions to interface with a SAFE server"""

SAFE_HTTP_POST_SUCCESS = 200
SAFE_RESULT_FIELD = "result"
SAFE_MESSAGE_FIELD = "message"
SAFE_SUCCESS = "succeed"

def hash_key(keyName):
    """ create a safe-compatible hash string of a public key file."""

    try:
        f = open(keyName, 'rb')
    except IOError:
        raise SafeException(f"Unable to read key file {keyName}")

    with f:
        try:
            r = RSA.importKey(f.read(), passphrase='')
        except:
            raise SafeException(f"Unable to parse key file: {keyName}")

        s = hashlib.sha256()
        s.update(r.exportKey(format='DER'))
        encoded = base64.urlsafe_b64encode(s.digest())

        return encoded.decode('utf-8')


def post_to_safe(*, headUrl, endpoint, principal, listOfParams):
    """ post a call to SAFE """
    params = {"principal": principal, "methodParams": listOfParams}
    #print(f"Parameters for {headUrl}{endpoint} are {params}")
    try:
        resp = requests.post(headUrl + endpoint, json=params)
    except Exception as e:
        raise SafeException(f"Connection error")

    if resp.status_code != SAFE_HTTP_POST_SUCCESS:
        raise SafeException(f"Unable to post due to error: {resp.status_code}")

    json = resp.json()
    if json[SAFE_RESULT_FIELD] != SAFE_SUCCESS:
        raise SafeException(f"POST failed due to error: {json[SAFE_MESSAGE_FIELD]}")
    return json[SAFE_MESSAGE_FIELD]

def post_raw_id_set(*, headUrl, principal):
    """ post a raw ID set of the principal """
    return post_to_safe(headUrl=headUrl, endpoint='postRawIdSet', principal=principal,
                      listOfParams=[principal])

def post_per_flow_rule(*, headUrl, principal, flowId):
    """ post a single per-workflow rule """
    return post_to_safe(headUrl=headUrl, endpoint='postPerFlowRule', principal=principal,
                      listOfParams=[flowId])

def post_two_flow_data_owner_policy(*, headUrl, principal, dataset, wf1, wf2):
    """ post a ruleset for two workflows associated with the dataset """
    return post_to_safe(headUrl=headUrl, endpoint='postTwoFlowDataOwnerPolicy', principal=principal,
                      listOfParams=[dataset, wf1, wf2])

class SafeException(Exception):
    """ SAFE-related exception. Behaves like Exception """
    pass


RESEARCH_APPROVAL_ID = '6c759d8a-311f-4f6d-ae7f-ffe8cf6d64a8'
INFRASTRUCTURE_APPROVAL_ID = '7087d9eb-466f-4dcb-bd23-77b67d1c69db'
DATASET_ID = '37af8d97-f7fd-456e-b6e8-d0a3292d56fc'
PUBLIC_KEY_PATH = '/Users/ibaldin/workspace-impact/impact-docker-images/safe-server/example-impact/principals/wp/wp1.pub'
SAFE_URL = 'http://localhost:7777/'



#!/usr/bin/env python

import pyforms
from wp_dso_publish.app_gui import AppGUI
from wp_dso_publish import local_settings
from confapp import conf

conf += local_settings

# Execute the application
if __name__ == "__main__":   pyforms.start_app(AppGUI, geometry=[100, 100, 500, 700])




#!/usr/bin/env python

__author__ = "Ilya Baldin"
__version__ = "0.1"
__maintainer__ = "Ilya Baldin"


import uuid

from pyforms.basewidget import BaseWidget
from pyforms.controls import ControlText
from pyforms.controls import ControlButton
from pyforms.controls import ControlTextArea
from pyforms.controls import ControlFile

from PyQt5.QtWidgets import QApplication, QMainWindow, QMessageBox
from PyQt5 import QtWidgets

from wp_dso_publish import safe_helper
from os import path
import sys
import pkg_resources

"""
Displays appropriate forms for a Data Provider/DataSet Owner to issue GUIDs
to workflows and a dataset and register the policy for the dataset with the
associated SAFE server.
"""


class AppGUI(BaseWidget):

    _safe_default_url = 'http://localhost:7777/'
    _saved_settings = pkg_resources.resource_filename(__name__, 'saved-settings.py')
    _saved_settings_module = 'wp_dso_publish.saved-settings'

    def __init__(self):
        super(AppGUI, self).__init__('Dataset Policy Registration')

        # merge settings
        if path.exists(self._saved_settings):
            from confapp import conf
            conf += self._saved_settings_module
            confLoaded = True
        else:
            confLoaded = False

        # form fields
        self._wp1 = ControlText('Research Approval Workflow ID      ')
        if confLoaded and hasattr(conf, 'RESEARCH_APPROVAL_ID'):
            self._wp1.value = conf.RESEARCH_APPROVAL_ID
        self._wp1gen = ControlButton('Generate')
        self._wp2 = ControlText('Infrastructure Approval Workflow ID')
        if confLoaded and hasattr(conf, 'INFRASTRUCTURE_APPROVAL_ID'):
            self._wp2.value = conf.INFRASTRUCTURE_APPROVAL_ID
        self._wp2gen = ControlButton('Generate')
        self._ds = ControlText('Dataset ID                         ')
        if confLoaded and hasattr(conf, 'DATASET_ID'):
            self._ds.value = conf.DATASET_ID
        self._dsgen = ControlButton('Generate')
        self._safeURL = ControlText('SAFE Server URL', default=self._safe_default_url,
                                    helptext='Root URL of the SAFE server')
        if confLoaded and hasattr(conf, 'SAFE_URL'):
            self._safeURL.value = conf.SAFE_URL
        self._safePubKeyPath = ControlFile('SAFE Public Key Path')
        if confLoaded and hasattr(conf, 'PUBLIC_KEY_PATH'):
            self._safePubKeyPath.value = conf.PUBLIC_KEY_PATH
        self._push = ControlButton('Push Combined Policy to SAFE')
        self._save = ControlButton('Save Settings')
        self._quit = ControlButton('Quit')
        self._results = ControlTextArea()
        self._results.readonly = True
        self._results.autoscroll = True

        # button actions
        self._wp1gen.value = self._getGUIDClosure(self._wp1)
        self._wp2gen.value = self._getGUIDClosure(self._wp2)
        self._dsgen.value = self._getGUIDClosure(self._ds)
        self._push.value = self.__pushToSafe
        self._save.value = self.__save
        self._quit.value = self.__quit

        # formset layout
        self.formset = [{'B. SAFE Parameters': ['_safeURL', '_safePubKeyPath'],
                        'A. Workflows and Datasets': [('_wp1', '_wp1gen'), ('_wp2', '_wp2gen'), ('_ds', '_dsgen')]},
                        '=',
                        ('_quit', '_save', '_push'), '=', ('_results')]

    def _getGUIDClosure(self, field):
        """ couldn't find a way to determine the field for which the button is pressed """

        def genGUID(self):
            field.value = uuid.uuid4().__str__()

        return genGUID

    @staticmethod
    def _findMainWindow():
        """ find the main window of the app """
        app = QApplication.instance()
        for widget in app.topLevelWidgets():
            if isinstance(widget, QMainWindow):
                return widget
        return None

    @staticmethod
    def _resizeWindow():
        win = AppGUI._findMainWindow()
        if win is not None:
            sizePolicy = QtWidgets.QSizePolicy(
                QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
            win.setSizePolicy(sizePolicy)
            win.resize(800, 600)
            #win.show()
        else:
            print("Unable to find main window")

    @staticmethod
    def _warningWindow(message, explain):
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Critical)
        msg.setText("Error: " + message)
        msg.setInformativeText(explain)
        msg.setWindowTitle("Error")
        msg.exec_()

    @staticmethod
    def _infoWindow(message, explain):
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Information)
        msg.setText(message)
        msg.setInformativeText(explain)
        msg.setWindowTitle("Information")
        msg.exec_()

    def __save(self):
        with open(self._saved_settings, 'w') as f:
            if len(self._wp1.value) != 0:
                f.write(f"RESEARCH_APPROVAL_ID = '{self._wp1.value}'\n")
            if len(self._wp2.value) != 0:
                f.write(f"INFRASTRUCTURE_APPROVAL_ID = '{self._wp2.value}'\n")
            if len(self._ds.value) != 0:
                f.write(f"DATASET_ID = '{self._ds.value}'\n")
            if len(self._safePubKeyPath.value) != 0:
                f.write(f"PUBLIC_KEY_PATH = '{self._safePubKeyPath.value}'\n")
            if len(self._safeURL.value) != 0:
                f.write(f"SAFE_URL = '{self._safeURL.value}'\n")
        self._infoWindow("Settings Saved", "All settings have been saved and will be loaded "
                                           "next time you start the program")

    def __quit(self):
        sys.exit(0)

    def __pushToSafe(self):
        """ post the necessary rules"""

        # hash the pulic key of the principal
        try:
            principal = safe_helper.hash_key(self._safePubKeyPath.value)
        except safe_helper.SafeException as e:
            AppGUI._warningWindow(e.__str__(), "Please use the 'B' tab to fill the parameters")
            return

        if len(self._wp1.value) == 0 or \
                len(self._wp2.value) == 0 or \
                len(self._ds.value) == 0:
            AppGUI._warningWindow("Workflow or Dataset IDs are not unique",
                                  "Please specify or generate unique IDs for workflows "
                                  "and the dataset using the 'A' tab")
            return

        # create complete IDs for WF1, WF2 and DS
        wf1_id = ":".join([principal, self._wp1.value])
        wf2_id = ":".join([principal, self._wp2.value])
        dataset_id = ":".join([principal, self._ds.value])

        res1 = res2 = res3 = res4 = None

        try:
            res1 = safe_helper.post_raw_id_set(headUrl=self._safeURL.value, principal=principal)
            res2 = safe_helper.post_per_flow_rule(headUrl=self._safeURL.value, principal=principal, flowId=wf1_id)
            res3 = safe_helper.post_per_flow_rule(headUrl=self._safeURL.value, principal=principal, flowId=wf2_id)
            res4 = safe_helper.post_two_flow_data_owner_policy(headUrl=self._safeURL.value, principal=principal,
                                                          dataset=dataset_id, wf1=wf1_id, wf2=wf2_id)
        except safe_helper.SafeException as e:
            AppGUI._warningWindow(e.__str__(), f"Unable to post to SAFE server {self._safeURL.value}")
            self._results.value = "There was an error communicating with SAFE server"

        if self._results.value is None or self._results.value == "":
            # output the results needed by the user for Notary Service
            self._results.value = f"All policies have been posted to the SAFE server. \n\
Please take note of the following identifiers for the Notary Service: \n\n\
Research Approval Workflow ID: {wf1_id}\n\n\
Infrastructure Approval Workflow ID: {wf2_id}\n\n\
Dataset ID: {dataset_id}\n\n\n\
For debugging purposes here are SAFE identifiers for the posted policies:\n\
postRawIdSet = {res1}\n\
postPerFlowRule (Research Approval) = {res2}\n\
postPerFlowRule (Infrastructure Approval) = {res3}\n\
postTwoFlowDataOwnerPolicy = {res4}"



#!/usr/bin/env python

import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='wp_dso_publish',
    version='0.28',
    packages=[
        'wp_dso_publish'
    ],
    package_dir={'wp_dso_publish': 'wp_dso_publish/'},
    package_data={'wp_dso_publish': ['style.css', 'local_settings.py']},
    scripts=['wp-dso-publish.py'],
    url='https://github.com/RENCI-NRIG/impact-utils/tree/master/wp_dso_publish',
    license='MIT',
    long_description=long_description,
    author='ibaldin',
    author_email='ibaldin@renci.org',
    description='Utility to push SAFE policies for a specific dataset.',
    classifiers=[
	    "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
      'PyForms-GUI==4.902',
      'pycryptodome==3.8.2',
      'requests==2.22.0',
    ],
)
